/* Factor & Extension Analysis Program  (SAS version) */

/* primary sources:
   Gorsuch, R. L. (1997a). Exploratory factor analysis: Its role
   in item analysis. Journal of Personality Assessment, 68, 532-560

   Gorsuch, R. L. (1997b). New procedure for extension analysis in
   exploratory factor analysis. Educational and Psychological
   Measurement, 57, 725-740 */

/* To specifiy your options, repond to the queries at the
   start of the program */


/* Prepare a raw data matrix for analysis,
   where rows = cases, & columns = variables;
   Cases with missing values are not permitted in the data file;
   The core variables must be the first x variables in the data file;
   The extension variables must appear after (to the right of)
   the core variables in the data file */

/* For the trial analyses, the program was set to read the raw data
   file named "trial.dat", and then process this data file within
   PROC IML -- see the "USE" command near the start of the program.
   Alternatively, the READ or EDIT statements (PROC IML commands)
   can also be used to read a data file.  */

/* Make sure to indicate the location of the "trial.dat" file on
   your Hard Drive. */

data trial;  infile 'Macintosh HD:Latest:ExtensionAnalysis:trial.dat'; input v1-v18;


options nocenter nodate nonumber  linesize=100 pagesize=500;  title;
proc iml;
reset noname;

/* Enter or read a raw data matrix, where rows = cases, & columns = variables
   Use the following name for the raw data matrix: "data".
   Cases with missing values are not permitted in the data file. */

use trial;
read all var _num_ into data;


/* How many core variables are there?
   The core variables must be the first x variables in the data file */
core = 14;

/* How many extension variables are there?
   The extension variables must appear after (to the left of) the core
   variables in the data file */
ext  = 4;

/* What kind of extension analysis do you want to run?
   Enter "1" for regular extension analysis
   Enter "2" for higher order factor/extension analysis */
kind = 2;

/* Specify a procedure for determining the number of factors:
   Enter "1" for # of salient loadings >= abs(.40)  (Gorsuch, 1997, J Pers Assess, p 545)
   Enter "2" for parallel analysis for principal components
             (Zwick & Velicer, 1986, Psych Bull, 99, p 432)
   Enter "3" for parallel analysis for principal axis / common factors
   Enter "4" for Velicer's MAP test (Velicer, 1976, Psychometrika, 41, 321-327)
   Enter "5" for Standard Error Scree test (Zoski & Jurs, 1996, EPM, p 443)
   Enter "6" for number of eigenvalues > 1
   Enter "7" for a user-selected number of factors (& see the next command) */
roottest = 1;

/* If "7" was specified for roottest, then enter the desired number of factors */
nfactors = 3;

/* If "7" was specified for roottest, then enter the desired number of factors
   for the higher order analysis (if requested) */
nfactsho = 1;

/* Specify a factor extraction procedure:
   Enter "1" for common factor analysis
   Enter "2" for principal components
   Enter "3" for maximum likelihood factor analysis
   Enter "4" for common image analysis  */
extract = 1;

/* Specify a rotation procedure:
   Enter "1" for promax
   Enter "2" for varimax     (not appropriate for higher order analyses)
   Enter "3" for No rotation (not appropriate for higher order analyses) */
rotate = 1;


/* End of required user specifications */


/* The following additional specifications can also be made (or ignored) */

/* Parallel analysis: Enter the # of random data sets */
Ndatsets = 10;

/* Parallel analysis: Enter the desired percentile eigenvalue (default = 95th) */
percent = 95;

/* Number of Salient Loadings:
   Enter the minimum abs value for a "salient" loading (default = .40) */
salvalue = .40;

/* Number of Salient Loadings:
  Enter the minimum # of salient loadings for there to be a "factor" (default = 3) */
numsals = 3;

/* Common Factor analysis: maximum number of iterations (default = 30) */
itercfa = 3;

/* Maximum Likelihood Factor analysis: maximum number of iterations (default = 30) */
iterml = 30;

/* Maximum Likelihood Factor analysis: tolerance for loadings (default = .001) */
tolerml = .001;

/* Promax rotation: the power value (default = 3) */
ppower = 3;

/* Promax rotation:
   Enter "1" for the structure matrix (default, & recommended by Gorsuch, 1997, p 556)
   Enter "2" for the pattern matrix  */
loadmat = 1;

/* End of optional specifications */


cases = nrow(data);
nvars = ncol(data);

print , "Extension Factor Analysis";
print , "Number of Cases in the Raw Data Matrix = ", cases;
print , 'Total Number of Variables in the Raw Data Matrix = ', nvars;
print , 'Number of Core Variables = ', core;
print , 'Number of Extension Variables = ', ext;

/* Improper specification warnings */
if ( (core+ext) > nvars)    then  do;
print , 'WARNING: More core and/or extension variables were specified than';
print , 'there are variables in the raw data matrix: Expect error messages';
end;
if ( nvars > (core+ext) )  then do;
print, 'WARNING: There are more variables in the raw data';
print  'than there are core & extension variables.  The data ';
print  'matrix was trimmed to core+extension variables.';
data = data[,1:(core+ext)];
nvars = ncol(data);
end;


if roottest = 1 then; print, 'Test for Number of Factors  =  # Salient Loadings';
if roottest=2 then;print,'Test for # Factors= Parallel Analysis of Prin Components';
if roottest=3 then;print,'Test for # Factors= Parallel Analysis of Common Factors';
if roottest = 4 then; print, "Test for Number of Factors  =  Velicer's MAP test";
if roottest = 5 then; print, 'Test for Number of Factors  =  Standard Error Scree';
if roottest = 6 then; print, 'Test for Number of Factors  =  # Eigenvalues > 1';
if roottest = 7 then; print, 'Test for Number of Factors  =  User-Specified';

if extract = 1  then; print 'Factor Extraction Procedure  =  Common Factor Analysis';
if extract = 2  then; print 'Factor Extraction Procedure  =  Principal Components';
if extract = 3  then; print 'Factor Extraction Procedure  =  Maximum Likelihood';
if extract = 4  then; print 'Factor Extraction Procedure  =  Image Analysis';

if rotate = 1   then; print 'Rotation Procedure  =  Promax';
if rotate = 2   then; print 'Rotation Procedure  =  Varimax';
if rotate = 3   then; print 'Rotation Procedure  =  No Rotation';


/* row sums module */
start rsum(matname);
rsums =j(nrow(matname),1);
do rows = 1 to nrow(matname);
dumr = matname[rows,];
rsums[rows,1]=sum(dumr);
end;
return(rsums);
finish;

/* collumn sums module */
start csum(matname);
csums =j(1,ncol(matname));
do cols = 1 to ncol(matname);
dumc = matname[,cols];
csums[1,cols]=sum(dumc);
end;
return(csums);
finish;

/* setdiag to a collumn vector  module */
start setdiag(matname,vector);
do ii = 1 to nrow(matname);
matname[ii,ii] = vector[ii,1];
end;
finish;

/* number of salient loadings module (test for the number of factors)
   ref = Gorsuch 1997 J Pers Assess p 545 */
start salient;
run nevalgt1; nfactors = nfneval1;
run cfa;
if nfactors > 1 then; run varimax;
rowmax = j( nrow(lding),1,-999);
do loopr = 1 to nrow(lding);
rowmax[loopr,1] = max(abs(lding[loopr,]));
end;
nfSAL = 0;
do lupec = 1 to ncol(lding);
nsalynts = 0;
do luper = 1 to nrow(lding);
if (abs(lding[luper,lupec]) >= salvalue  &  abs(lding[luper,lupec]) = rowmax[luper,1]) then;
nsalynts = nsalynts + 1;
end;
if nsalynts >= numsals then; nfSAL = nfSAL + 1;
end;
finish;

/* Parallel Analysis for Principal Components module */
start paralPCA;
Ncases   = cases;
Numvars  = nrow(r);
/* computing random data correlation matrices & eigenvalues */
evals = j(Numvars,ndatsets,-9999);
nm1 = 1 / (ncases-1);
do nds = 1 to ndatsets;
x = normal(j(ncases,Numvars));
vcv = nm1 * (t(x)*x - ((t(x[+,])*x[+,])/ncases));
d = inv(diag(sqrt(vecdiag(vcv))));
evals[,nds] = eigval(d * vcv * d);
end;
/* identifying the eigenvalues corresponding to the desired percentile */
num = round((percent*ndatsets)/100);
prcntyle =  t(1:Numvars);
do root = 1 to Numvars;
ranks = rank(evals[root,]);
do col = 1 to ndatsets;
if (ranks[1,col] = num) then do;
prcntyle[root,1] = evals[root,col];
col = ndatsets;
end;end;end;
nfPARPCA = 0;
do loopr = 1 to Numvars;
if ( evals1[loopr,1] < prcntyle[loopr,1] ) then do;
nfPARPCA = loopr - 1;
loopr = Numvars;
end;end;
finish;

/* Parallel Analysis for Principal Axis / Common Factors module */
start paralPAF;
Ncases   = cases;
Numvars  = nrow(r);
/* SMCs & eigenvalues of the real correlation matrix */
rdum1 = r;
smc = 1 - (1 / vecdiag(inv(rdum1)) );
run setdiag(rdum1,smc);
evalsr = eigval(rdum1);
/* computing random data correlation matrices & eigenvalues */
evals = j(Numvars,ndatsets,-9999);
nm1 = 1 / (ncases-1);
do nds = 1 to ndatsets;
x = normal(j(ncases,Numvars));
vcv = nm1 * (t(x)*x - ((t(x[+,])*x[+,])/ncases));
d = inv(diag(sqrt(vecdiag(vcv))));
rdum2 = d * vcv * d;
smc = 1 - (1 / vecdiag(inv(rdum2)) );
run setdiag(rdum2,smc);
evals[,nds] = eigval(rdum2);
end;
/* identifying the eigenvalues corresponding to the desired percentile */
num = round((percent*ndatsets)/100);
prcntyle =  t(1:Numvars);
do root = 1 to Numvars;
ranks = rank(evals[root,]);
do col = 1 to ndatsets;
if (ranks[1,col] = num) then do;
prcntyle[root,1] = evals[root,col];
col = ndatsets;
end;end;end;
nfPARPAF = 0;
do loopr = 1 to Numvars;
if ( evalsr[loopr,1] < prcntyle[loopr,1] ) then do;
nfPARPAF = loopr - 1;
loopr = Numvars;
end;end;
finish;

/*  Velicer's Minimum Average Partial (MAP) Test module */
start velmap;
call eigen (eigvals,eigvects,r);
loadings = eigvects * sqrt(diag(eigvals));
fm = j(nrow(r),2,-9999);
fm[1,2] = (ssq(r) - ncol(r))/(ncol(r)*(ncol(r)-1));
do m = 1 to ncol(r) - 1;
a = loadings[,1:m];
partcov = r - (a * t(a));
d = diag( 1 / (sqrt(vecdiag(partcov))) );
pr = d * partcov * d;
fm[m+1,2] = (ssq(pr)-ncol(r)) / (ncol(r)*(ncol(r)-1));
end;
/* identifying the smallest fm value & its location (= the of factors) */
minfm = fm[1,2];
nVEL = 0;
do s = 1 to nrow(fm);
fm[s,1] = s - 1;
if ( fm[s,2] < minfm ) then do;
minfm = fm[s,2];
nfVEL = s - 1;
end;
end;
finish;

/* Standard Error Scree test (Zoski & Jurs, 1996, EPM, p 443)  module*/
start sescree;
evals = eigval(r);
nevals = nrow(evals);
rootnum = t(1:nevals);
sesdata = ( rootnum || evals );
nfSCREE = 0;
arbiter = 1 / nevals;
do loop = 1 to (nevals-2);
y = sesdata[loop:nevals,2] ;
n = nrow(y);
x = ( j(n,1,1) || sesdata[loop:nevals,1] );
b = inv( t(x) * x) * t(x) * y;
pred = x * b;
sse = sum ( (y - pred)##2 );
sderest = sqrt( sse / (n - 2) );
if sderest > arbiter then;  nfSCREE = nfSCREE + 1;
end;
finish;

/* Number of eigenvalues > 1 module */
start nevalgt1;
eigvals = eigval(r);
nfneval1 = 0;
do nev = 1 to nrow(eigvals);
if eigvals[nev,:] > 1 then;  nfneval1 = nfneval1 + 1;
end;
finish;

/* CFA / PAF module -- from Bernstein p 189; smc = from Bernstein p 104 */
start cfa;
rcfa = r;
smc = 1 - (1 / vecdiag(inv(rcfa)) );
do iter = 1 to (itercfa + 1);
/* putting smcs on the main diagonal of */
run setdiag(rcfa,smc);
eigvals  = eigval (rcfa);
eigvects = eigvec (rcfa);
lding = eigvects[,1:nfactors] * sqrt(diag( (eigvals[1:nfactors,]) ));
communal = rsum( lding##2 );
if  max(max(abs(communal-smc))) < .001 then; iter = itercfa + 1;
if  max(max(abs(communal-smc))) > .001  & iter < itercfa then;  smc=communal;
end;
finish;

/* PCA module */
start pca;
eigvals  = eigval (r);
eigvects = eigvec (r);
lding = eigvects[,1:nfactors] * sqrt(diag( (eigvals[1:nfactors,]) ));
finish;

/* Maximum Likelihood Factor Analysis -- Morrison (1976, p 308-309)  module */
start maxlike;
k = nfactors;
Rho = r;
p = nrow(Rho);
L = diag(eigval(Rho));
A = eigvec(Rho);
A1=A[,1:k]*sqrt(L[1:k,1:k]);
Uni=diag(vecdiag(Rho-A1*t(A1)));
Rh1=sqrt(inv(Uni))*(Rho-Uni)*sqrt(inv(Uni));
L = diag(eigval(Rh1));
A = eigvec(Rh1);
A1=sqrt(Uni)*A[,1:k]*sqrt(L[1:k,1:k]);
check=tolerml;
do i = 1 to iterml;
Uni=diag(vecdiag(Rho-A1* t(A1) ));
Rh1=sqrt(inv(Uni))*(Rho-Uni)*sqrt(inv(Uni));
L = diag(eigval(Rh1));
A = eigvec(Rh1);
A2=sqrt(Uni)*A[,1:k]*sqrt(L[1:k,1:k]);
if max(max(abs(A1-A2)))<check then; i= iterml;
A1=A2;
end;
lding = A1;
finish;

/* Image Factor Extraction module
   (Gorsuch 1983 p 113), Velicer (1974, Educ & Psych Meas, 34, p 564) */
start imagefa;
d =  diag( 1 / vecdiag(inv(r)) );
gvv = r + d * inv(r) * d - 2 * d;
r = gvv;
s = sqrt(d);
r2 = inv(s) *  gvv  * inv(s);
call eigen(eigvali,eigvect,r2);
l = eigvect[,1:nfactors];
dd = sqrt(diag(eigvali[1:nfactors,]));
lding = s * l * dd;
finish;


/* varimax module */
start varimax;
b=lding;
n = nrow(lding);
nf = ncol(lding);
hjsq=vecdiag(lding*t(lding));
hj=sqrt(hjsq);
bh=lding / (hj*(j(1,nf,1)));
Vtemp=n*rsum(csum(bh##4))-rsum(csum(bh##2)##2);
V0=Vtemp;
do loopit=1 to 10;
do loopi =1 to (nf-1);
  jl=loopi+1;
  do loopj=jl to nf;
      xj=lding[,loopi] / hj;
      yj=lding[,loopj] / hj;
      uj=xj # xj-yj # yj;
      vj=2*xj # yj;
      A=csum(uj);
      bigB=csum(vj);
      C=t(uj) * uj - t(vj) * vj;
      D=2 * t(uj) * vj;
      num=D-2*A*bigB/n;
      den=C-(A**2-bigB**2)/n;
      tan4p=num/den;
tan4p = num / den;
artanp = ( atan(abs(tan4p)) ) / 4;
if (num>0 & den>0) then;  phi = artanp;
if (num<0 & den<0) then;  phi =  -1 * (.7854 - artanp);
if (num<0 & den>0) then;  phi =  -1 * artanp;
if (num>0 & den<0) then;  phi =  .7854 - artanp;
angle = phi * 180 / 3.1415926;
      if (  abs(phi)>.00001  ) then do;
          bigXj=cos(phi)*xj+sin(phi)*yj;
          bigYj=-sin(phi)*xj+cos(phi)*yj;
          bj1=bigXj # hj;
          bj2=bigYj # hj;
          b[,loopi]=bj1;
          b[,loopj]=bj2;
          lding[,loopi]=b[,loopi];
          lding[,loopj]=b[,loopj];
      end;
  end;
end;
lding=b;
bh=lding / (hj*(j(1,nf,1)));
Vtemp=n*rsum(csum(bh##4))-rsum(csum(bh##2)##2);
V=Vtemp;
if ( abs(V-V0)<.0001 )  then;  loopit = 10;
if ( abs(V-V0)>=.0001 ) then;   V0=V;
end;
finish;

/* promax module */
start promax;
run varimax;
B = lding;
Bstar=B##ppower;
Tr=inv(t(B) *B)*t(B)*Bstar;
Tr=Tr*sqrt(inv(diag(vecdiag(t(Tr)*Tr))));
Tpp=inv(Tr);
Tpp=inv(sqrt(diag(vecdiag(Tpp*t(Tpp)))))*Tpp;
Sr=B*Tr;
Phip=Tpp*t(Tpp);
Sp=B*t(Tpp);
Pp=B*inv(Tpp);
Pr=B*inv(t(Tr));
Phir= t(Tr)*Tr;
if loadmat = 1 then; lding = Sp;
if loadmat = 2 then; lding = Pp;
finish;


/* correlation matrix */
zscores = j(cases, nvars, -999);
n   = nrow(data);
nm1 = n - 1;
do loopc = 1 to ncol(data);
mean = sum(data[,loopc]) / n;
sd = sqrt ( ssq((data[,loopc]- mean)) / nm1  );
zscores[,loopc] = (data[,loopc] - mean) / sd;
end;
rdata = (1 / (nrow(data)-1) ) * ( t(zscores) * zscores );


rcore =  rdata[1:core,1:core];
r = rcore;
evals1 = eigval(r);


/* Number of Factors */
if roottest = 1 then do; run salient;     nfactors = nfSAL;    end;
if roottest = 2 then do; run paralPCA;    nfactors = nfPARPCA; end;
if roottest = 3 then do; run paralPAF;    nfactors = nfPARPAF; end;
if roottest = 4 then do; run velmap ;     nfactors = nfVEL;    end;
if roottest = 5 then do; run sescree;     nfactors = nfSCREE;  end;
if roottest = 6 then do; run nevalgt1;    nfactors = nfneval1; end;
warnnf1 = 0;
if nfactors = 0 then do;                     nfactors = 1; warnnf1 = 1; end;

/* Factor Extraction */
if extract = 1 then; run cfa;
if extract = 2 then; run pca;
if extract = 3 then; run maxlike;
if extract = 4 then; run imagefa;

/* Rotation */
if nfactors = 1 then; rff = 1;
if nfactors  > 1 then do;
if rotate = 1 then do; run promax;   rff = Phip;end;
if rotate = 2 then   ; run varimax;
end;


nfactrs1 = nfactors;

/* for regular extension analysis */
/* supermatrix: core variables, extension variables, & the factor loadings */
sup = j( (ncol(rdata)+nfactors) , (ncol(rdata)+nfactors), 0 );
sup[1:nrow(rdata),1:ncol(rdata)] = rdata;
sup[ (nrow(sup)-(nfactors-1)):nrow(sup), 1:nrow(lding) ] = t(lding[1:core,]);


/* correlations between extension variables & the factors */
ppp = j( nrow(sup) , ncol(sup), 0 );
resid = sup;
do lupe  = 1 to core;
do luper = lupe to nrow(sup);
ppp[luper,lupe] = resid[luper,lupe] / sqrt(resid[lupe,lupe]);
end;
resid =  resid - ppp[,lupe] * t(ppp[,lupe]);
end;
pvo = ppp[1:core,1:core];
peo = ppp[core+1:(core+ext),1:core];
pfo = ppp[ (core+ext+1):(nrow(ppp)) , 1:core];
sef1 = peo * t(pfo);



/* for higher order factor/extension analysis */
if ( kind = 2 & nfactrs1 > 1 & rotate = 1) then do;
lding1 = lding;
core1  = core;
ext1   = ext;

r = rff;
evals2 = eigval(r);

/* Number of Factors */
if roottest = 1 then do; run salient;     nfactors = nfSAL;    end;
if roottest = 2 then do; run paralPCA;    nfactors = nfPARPCA; end;
if roottest = 3 then do; run paralPAF;    nfactors = nfPARPAF; end;
if roottest = 4 then do; run velmap ;     nfactors = nfVEL;    end;
if roottest = 5 then do; run sescree;     nfactors = nfSCREE;  end;
if roottest = 6 then do; run nevalgt1;    nfactors = nfneval1; end;
if roottest = 7 then   ;                  nfactors = nfactsho;
warnnf2 = 0;
if nfactors = 0 then do;                     nfactors = 1; warnnf2 = 1;end;

/* Factor Extraction */
if extract = 1 then; run cfa;
if extract = 2 then; run pca;
if extract = 3 then; run maxlike;
if extract = 4 then; run imagefa;

/* Rotation */
if nfactors  > 1 then do;
if rotate = 1 then; run promax;
end;

/* supermatrix: core variables, extension variables, & the factor loadings */
sup = j( (nfactrs1+core+ext+nfactors) , (nfactrs1+core+ext+nfactors), 0 );

/* the core variables for the higher order analysis =
   the factor intercorrelations */
sup[1:nrow(rff),1:ncol(rff)] = rff;

/* the first set of extension variables =
   the variables that were in the first factor analysis */
sup[ (nfactrs1+1):(nfactrs1+core), 1:(ncol(lding1)) ] = lding1;

/* the possible additional extension variables =
   the variables that were not in the first factor analysis */
if ext > 0 then;
sup[ (nfactrs1+core+1):(nfactrs1+core+ext) , (1:nfactors) ] =
    rdata[ (core+1):(core+ext), (1:nfactors) ];

/* the loadings (correlations) from the higher order factor analysis */
sup[ (nrow(sup)-(nfactors-1)):nrow(sup), (1:nrow(lding)) ] = t(lding);

ext = core + ext;
core = nfactrs1;


/* correlations between extension variables & the higher order factors */
ppp = j( nrow(sup) , ncol(sup), 0 );
resid = sup;
do lupe  = 1 to core;
do luper = lupe to nrow(sup);
ppp[luper,lupe] = resid[luper,lupe] / sqrt(resid[lupe,lupe]);
end;
resid =  resid - ppp[,lupe] * t(ppp[,lupe]);
end;
pvo = ppp[1:core,1:core];
peo = ppp[core+1:(core+ext),1:core];
pfo = ppp[ (core+ext+1):(nrow(ppp)) , 1:core];
sef = peo * t(pfo);

end;


if ( kind = 1 | (kind = 2 & nfactrs1 = 1) | (kind = 2 & rotate > 1) ) then do;
print, 'Eigenvalues for the Core Variables:';
print  ( t(1:nrow(evals1)) || evals1 ) [format=7.5];
if warnnf1 = 1 then do;
print , 'WARNING: Zero factors were found in the data.';
print , '         The number of factors was therefore set at 1.';
end;
print, 'Number of Factors = ' , nfactors;
if nfactrs1 = 1 then;
print 'WARNING: There was only one factor, rotation not performed';
fs={F F F F F F F F F F F F F F F F F F F F F F F F F F F F F F};
labels = ( "Item" || fs[1,1:nfactrs1] );
if (rotate = 1 & nfactrs1 > 1) then;
print, 'Factor Intercorrelations:';
print ( t(1:nrow(rff)) || rff ) [colname=labels format=5.2];
print, 'Core Variable Loadings on the Factors:';
print  (t(1:nrow(lding)) || lding) [colname=labels format=5.2];
if ext > 0 then;
print, 'Extension Variable Correlations with the Factors';
print  ( t(1:nrow(sef1)) || sef1)  [colname=labels format=5.2];
end;


/* Warning message */
if ( kind = 2 & nfactrs1 = 1 ) then do;
print, 'WARNING: Higher Order Extension Factor Analysis was requested, but there was';
print  '         only one factor in the lower-order data.';
print  '         The higher order analyses were not performed.';
end;

/* Warning message */
if (kind = 2 & rotate > 1) then do;
print, 'WARNING: Higher Order Factor/Extension Analysis was requested, as was';
print  '         a ROTATE option that does not produce correlations between';
print  '         between factors.  The higher order analyses were not performed.';
end;


if ( kind = 2 & nfactrs1 > 1 & rotate = 1) then do;
print, 'Eigenvalues for the First Set of Core Variables:';
print  ( t(1:nrow(evals1)) || evals1)  [format=7.5];
print, 'Number of Factors in the First Set of Core Variables = ' , nfactrs1;
print, 'Eigenvalues for the Higher Order Factor Analysis:';
print  ( t(1:nrow(evals2)) || evals2)  [format=7.5];
if warnnf2 = 1 then do;
print , 'WARNING: Zero factors were found in the higher order data.';
print   '         The number of factors was therefore set at 1.';
end;
print, 'Number of Factors for the Higher Order Factor Analysis = ' , nfactors;
if nfactors = 1 then do;
print  'WARNING: There was only one factor in the higher order data.';
print  '         Rotation was not performed.';
end;
fs={F F F F F F F F F F F F F F F F F F F F F F F F F F F F F F};
hos={HO HO HO HO HO HO HO HO };
labels = ( "Item" || fs[1,1:nfactrs1] || hos[1,1:nfactors] );
print, 'Factor Intercorrelations from the First Factor Analysis, ';
print  'and (separated by a collumn of zeros) the';
print  'loadings on the Higher Order Factor(s):';
print  ( t(1:nrow(lding)) || rff || lding) [colname=labels format=5.2];
print, 'Core Variable Loadings on the Lower Order Factors, ';
print  'and (separated by a collumn of zeros) their  ';
print  'Correlations with the Higher Order Factor(s):';
print ( t(1:nrow(lding1)) || lding1 || sef[1:core1,] )
      [colname=labels format=5.2];
if ext1 > 0 then;
print, 'Extension Variable Correlations with the Lower & Higher Order Factor(s)';
print ( t(1:ext1) || sef1 || sef[core1+1:nrow(sef),] )
      [colname=labels format=5.2];
end;

quit;
